"""
Exceptions module
"""


class ImportException(Exception):
    "Raised when there was import exception in print_seeker"


class ParentModuleNotFoundException(Exception):
    "Raised when there was import exception in print_seeker"
